<?php
if(!isset($config))
	die();
// sets default actions
add_action('start','verify_setup');
add_action('start','login_actions');
add_action('start','set_user_cookie');

function login_actions(){ // sets default plugins -- remove any you don't want
	add_action('signup','require_activation');
	add_action('signup','add_join_date');
	add_action('signup','signup_notification');
	add_action('activate','add_activation_date');
	add_action('login','login_notification');
	add_action('login','merge_cookie');
}

function require_activation($data){ // requires the user to activate their account
	global $config,$db;
	$key=md5(uniqid(rand(),true));
	$db->insert("usermeta",array('user'=>$data['id'],'key'=>'activate','value'=>$key));
	sendmail($data['email'],'New Account at '.$config['site']['name'],parse_template('email/activation',array('link'=>$config['pages']['activate'].'/'.$key),false));
}

function signup_notification($data){ // sends an e-mail notification when a user signs up
	global $config;
	sendmail($config['site']['email'],'New User at '.$config['site']['name'],parse_template('email/signup-notification',$data,false));
}

function add_join_date($data){ // sends an e-mail notification when a user signs up
	global $db;
	$db->insert("usermeta",array('user'=>$data['id'],'key'=>'joined','value'=>time()));
}

function add_activation_date($data){ // sends an e-mail notification when a user signs up
	global $db;
	$db->insert("usermeta",array('user'=>$data['id'],'key'=>'activated','value'=>time()));
}

function login_notification($data){ // sends an e-mail notification when a user logs in
	global $config;
	sendmail($config['site']['email'],'User Login on '.$config['site']['name'],parse_template('email/login-notification',$data,false));
}

function verify_setup(){ // returns true if everything is ready, creates tables and returns true if not
	global $db;
	if(!$db->ready||!users_table_setup()||!images_table_setup()||!hits_table_setup()||!usermeta_table_setup()||!imagemeta_table_setup()||!hitmeta_table_setup())
		die("Database not ready");
}

function users_table_setup(){ // used by verify_setup - checks to see if the users table is setup, creates it if not
	global $config,$db;
	return $db->query('SELECT id FROM users LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `users` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `name` VARCHAR('.$config['username']['max'].') NOT NULL, `password` VARCHAR(32) NOT NULL, `email` VARCHAR(64) NOT NULL) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function images_table_setup(){ // checks to see if the images table is setup, creates it if not
	global $config,$db;
	return $db->query('SELECT id FROM images LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `images` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `hash` VARCHAR(32) NOT NULL UNIQUE KEY) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function hits_table_setup(){ // checks to see if the hits table is setup, creates it if not
	global $config,$db;
	return $db->query('SELECT id FROM hits LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `hits` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `image` INT NOT NULL, `time` INT(11) NOT NULL, INDEX (`image`)) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function usermeta_table_setup(){ // used by verify_setup - checks to see if the usermeta table is setup, creates it if not
	global $db;
	return $db->query('SELECT id FROM usermeta LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `usermeta` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `user` INT NOT NULL, `key` VARCHAR(64) NULL, `value` TEXT NULL, INDEX (`user` ,`key`)) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function imagemeta_table_setup(){ // checks to see if the imagemeta table is setup, creates it if not
	global $db;
	return $db->query('SELECT id FROM imagemeta LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `imagemeta` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `image` INT NOT NULL, `key` VARCHAR(64) NULL, `value` TEXT NULL, INDEX (`image` ,`key`)) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function hitmeta_table_setup(){ // checks to see if the hitmeta table is setup, creates it if not
	global $db;
	return $db->query('SELECT id FROM hitmeta LIMIT 1;')!==false||$db->query('CREATE TABLE IF NOT EXISTS `hitmeta` (`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY, `hit` INT NOT NULL, `key` VARCHAR(64) NULL, `value` TEXT NULL, INDEX (`hit` ,`key`)) ENGINE=MyISAM DEFAULT CHARSET=utf8;')===0;
}

function user_id(){ // returns the user's id
	global $config;
	if(logged_in())
		return md5(sha1($_SESSION['user']['name']).$config['password']['salt']);
	elseif(!isset($_COOKIE[$config['site']['tracker']])){
		$set="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
		$user="";
		for($i=0;$i<32;$i++)
			$user.=$set[rand(0,strlen($set)-1)];
		return $user;
	}
	return strip($_COOKIE[$config['site']['tracker']]);
}

function merge_cookie(){ // adds the permanant userid to the image and hits table when the user logs in
	global $config,$db;
	if(isset($_COOKIE[$config['site']['tracker']])){
		$cookie=strip($_COOKIE[$config['site']['tracker']]);
		$id=user_id();
		$db->update("imagemeta",array('value'=>$id),array('key'=>'user','value'=>$cookie));
		$db->update("hitmeta",array('value'=>$id),array('key'=>'user','value'=>$cookie));
	}
}

function base62_encode($n){ // base 62-ish encoding
	$s="";$c="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_";
	$m=strlen($c);
	for($t=floor(log10($n)/log10($m));$t>=0;$t--){
		$a=floor($n/pow($m,$t));
		$s=$s.substr($c,$a,1);
		$n=$n-($a*pow($m,$t));
	}
	return $s;
}

function base62_decode($s){ // base 62-ish decoding
	$n=0;$c="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_";
	$m=strlen($c);
	$l=strlen($s)-1;
	for($t=0;$t<=$l;$t++)
		$n=$n+strpos($c,substr($s,$t,1))*pow($m,$l-$t);
	return $n;
}

function upload_form(){ // prints the upload form
	parse_template('forms/upload');
}

function caption_form(){ // prints the caption form
	parse_template('forms/caption');
}

function caption(){ // processes the caption form
	global $config,$db;
	if(!($id=$db->get_var($db->prepare('SELECT id FROM images WHERE hash=%s',strip($_POST['hash'])))))
		fail('No such image');
	if(user_id()!=get_meta('user',$id,'image'))
		fail('This image does not belong to you');
	if(get_meta('caption',$id,'image'))
		$db->update('imagemeta',array('value'=>html_chars(strip($_POST['caption']))),array('image'=>$id,'key'=>'caption'));
	else
		$db->insert('imagemeta',array('image'=>$id,'key'=>'caption','value'=>html_chars(strip($_POST['caption']))));
	fail('Caption saved');
}

function upload($file='',$fail='epic_fail'){ // processes the upload
	global $config,$db;
	if(!$file){
		if(!($file=$_FILES['image']['tmp_name']))
			$fail('Invalid image');
		if($_POST['MAX_FILE_SIZE']!=max_upload_size())
			$fail('Invalid filesize');
		if(intval($_FILES['image']['size'])>max_upload_size())
			$fail('Filesize too large');
		$file=$_FILES['image']['tmp_name'];
		$source=$_FILES['image']['name'];
	}
	else
		$source=$file;
	$uid=md5(uniqid(rand(),true));
	$upload=$config['folders']['uploads'].$uid;
	$thumb=$config['folders']['thumbnails'].$uid;
	if(!(createpng($file,$upload)&&createpng($file,$thumb,120,120)))
		$fail('Upload failed');
	$hash=md5(file_get_contents($upload));
	if($id=$db->get_var($db->prepare("SELECT id FROM images WHERE hash=%s",$hash))){
		unlink($thumb);
		unlink($upload);
		$fail('Image already exists: /'.base62_encode($id),'/'.base62_encode($id));
	}
	$db->insert('images',array(
		'hash'=>$hash
	));
	$id=$db->insert_id;
	$meta=array(
		'user'=>user_id(),
		'ip'=>$_SERVER['REMOTE_ADDR'],
		'created'=>time(),
		'caption'=>html_chars(basename($source)),
		'source'=>$source);
	foreach($meta AS $key => $value)
		if($value)
			$db->insert('imagemeta',array(
				'image'=>$id,
				'key'=>$key,
				'value'=>$value));
	do_action('new_image',array_merge(array('link'=>'http://'.$_SERVER['SERVER_NAME'].'/'.base62_encode($id),'image'=>'http://'.$_SERVER['SERVER_NAME'].'/thumbnails/'.base62_encode($id).'.png'),$meta));
	$_SESSION['create'][$id]=1;
	rename($thumb,dirname($thumb).'/'.$hash.'.png');
	rename($upload,dirname($upload).'/'.$hash.'.png');
	$fail('Upload successful: /'.base62_encode($id),'/'.base62_encode($id));
}

function upload_notification($data){ // sends an e-mail when a new picture is added
	global $config;
	sendmail($config['site']['email'],'New Image on '.$config['site']['name'],parse_template('email/new-image',$data,false));
}

function image(){ // pulls the image from the database
	global $config,$db,$image;
	$id=base62_decode($_GET['image']);
	$image=$db->get_row($db->prepare("SELECT * FROM images WHERE id=%d",$id));
	if(!$image)
		fail('No such image.','/');
	$result=$db->get_results($db->prepare("SELECT * FROM imagemeta WHERE image=%d",$id));
	foreach($result as $meta)
		$image->meta[$meta->key]=$meta->value;
	if(!isset($_SESSION['hash'][$id])&&$_GET['page']!='stats'&&$_GET['page']!='delete'&&!is_bot()){
		$_SESSION['hash'][$id]=md5(uniqid(rand(),true));
		if($image->meta['user']!=user_id()||$_SESSION['create'][$id])
			do_action('view_image',array('id'=>$id,'user'=>user_id(),'ip'=>$_SERVER['REMOTE_ADDR'],'referrer'=>strip($_SERVER['HTTP_REFERER']),'agent'=>$_SERVER['HTTP_USER_AGENT']));
	}
	if($_SESSION['create'][$id])
		unset($_SESSION['create'][$id]);
	if($_GET['page']=='uploads'||$_GET['page']=='thumbnails'){
		if($_GET['page']=='uploads'&&$_GET['hash']!=nonce($image->hash,3600))
			die();
		$image=$config['folders'][$_GET['page']].$image->hash.'.png';
		if(file_exists($image)){
			header('Content-type: image/png');
			echo file_get_contents($image);
		}
	}
	return;
}

function log_hit($data){ // logs the visit
	global $db;
	if(!$data['hash']&&!$data['referrer'])
		return;
	$db->insert("hits",array('image'=>$data['id'],'time'=>time()));
	$id=$db->insert_id;
	unset($data['id']);
	foreach($data AS $key => $value)
		if($value)
			$db->insert('hitmeta',array(
				'hit'=>$id,
				'key'=>$key,
				'value'=>$value));
}

function my_images(){ // pulls all the user's images and prints the list
	global $db;
	$images=$db->get_results($db->prepare("SELECT image AS id FROM imagemeta WHERE `key`='user' AND value=%s ORDER BY image DESC",strip(user_id())));
	foreach($images AS &$image){
		$result=$db->get_results($db->prepare("SELECT * FROM imagemeta WHERE image=%d",$image->id));
		foreach($result as $meta)
			$image->meta[$meta->key]=$meta->value;
		$image->hits=$db->get_var($db->prepare('SELECT count(id) FROM hits WHERE image=%d',$image->id));
	}
	parse_template('body/list',array('images'=>$images));
}

function set_user_cookie(){ // sets the tracking cookie on page load
	global $config;
	if(!isset($_COOKIE[$config['site']['tracker']])){
		$set="0123456789";
		$user="";
		for($i=0;$i<9;$i++)
			$user.=$set[rand(0,strlen($set)-1)];
		setcookie($config['site']['tracker'],$user,mktime(0,0,0,6,3,2025),'/',$_SERVER['HTTP_HOST'],false,false);
	}
}

function __($file,$data=''){ // builds the html page
	global $title, $keywords, $description, $robots;
	parse_template('body/header',array('title'=>$title,'keywords'=>$keywords,'description'=>$description,'robots'=>$robots));
	parse_template('pages/'.$file,$data);
	parse_template('body/footer');
}

function get_stats(){ // returns the image statistics
	global $config,$db,$image,$stats;
	$hits=$db->get_results($db->prepare('SELECT id, time FROM hits WHERE image=%d',$image->id));
	$stats=$ids=array();
	$days=min(ceil((time()-$image->meta['created'])/84600),60);
	$min=ceil((time()-86400*$days)/84600)*84600;
	$stats['day']=array_fill_keys(range(0,$days),0);
	foreach($hits AS $hit){
		$ids[]=$hit->id;
		if($hit->time>=$min)
			$stats['day'][(int)($days-ceil((time()-$hit->time)/84600)+1)]++;
	}
	$stats['days']=$days;
	$stats['total']=count($hits);
	$ips=$db->get_col("SELECT value FROM hitmeta WHERE `key`='ip' AND hit IN (".implode(',',$ids).")");
	$stats['unique']=$db->get_var("SELECT count(distinct value) FROM hitmeta WHERE `key`='user' AND hit IN (".implode(',',$ids).")");
	$referrers=$db->get_col("SELECT value FROM hitmeta WHERE `key`='referrer' AND hit IN (".implode(',',$ids).")");
	foreach($referrers AS &$referrer)
		$referrer=preg_replace('/(.*\/\/)(www\.)?([^\/]+)(\/)(.*)/','\3',$referrer);
	$stats['referrers']=array_count_values($referrers);
	if($stats['total']-count($referrers))
		$stats['referrers']['None']=$stats['total']-count($referrers);
	arsort($stats['referrers']);
}

function delete_image(){ // deletes the image if the user is the admin (first registered user) or image owner
	global $config,$db,$image;
	if($image->meta['user']!=user_id())
		fail('');
	$db->query($db->prepare('DELETE FROM hitmeta WHERE hit IN (SELECT id FROM hits WHERE image=%d)',$image->id));
	$db->query($db->prepare('DELETE FROM hits WHERE image=%d',$image->id));
	$db->query($db->prepare('DELETE FROM imagemeta WHERE image=%d',$image->id));
	$db->query($db->prepare('DELETE FROM images WHERE id=%d',$image->id));
	@unlink($config['folders']['thumbnails'].$image->hash.'.png');
	rename($config['folders']['uploads'].$image->hash.'.png',$config['folders']['trash'].$image->hash.'.png');
	fail(($image->meta['caption']?$image->meta['caption']:'Image '.$_GET['image']).' has been deleted.');
}

function max_upload_size(){ // returns the server's maximum upload size for upload form
	return min(to_bytes(ini_get('post_max_size')),to_bytes(ini_get('upload_max_filesize')));
}

function to_bytes($v){ // converts bytes to readable format
    $l=substr($v,-1);
    $bytes=substr($v,0,-1);
    switch(strtoupper($l)){
		case 'P':
			$bytes*=1024;
		case 'T':
			$bytes*=1024;
		case 'G':
			$bytes*=1024;
		case 'M':
			$bytes*=1024;
		case 'K':
			$bytes*=1024;
			break;
    }
    return $bytes;
}

function epic_fail($error,$redirect=''){ // used instead of fail for uploads
	die($error);
}

function createpng($src,$dest,$width=800,$height=800){ // converts, shrinks, and tweaks the image
	$data=@getimagesize($src);
    $img=@imagecreatefromstring(file_get_contents($src));
    if(!$img)
        return false;
    $sw=imagesx($img);
    $sh=imagesy($img);
    if($sw<$width&&$sh<$height){
        $w=$sw;
        $h=$sh;
    }elseif($sw>$sh){
        $w=$width;
        $h=floor(($sh*($height/$sw)));
    }elseif($sw<$sh){
        $w=floor($sw*($width/$sh));
        $h=$height;
    }elseif($sw==$sh){
        $w=$width;
        $h=$height;
    }
    $w=($w<1)?1:$w;
    $h=($h<1)?1:$h;
    $thumb=@imagecreatetruecolor($w,$h);
	if($data['mime']=="image/png"){
		imagealphablending($thumb,false);
		imagefill($thumb,0,0,imagecolorallocatealpha($thumb,0,0,0,127));
		imagesavealpha($thumb,true);
	}elseif($data['mime']=="image/gif"){
		$transparent=imagecolortransparent($img);
		if($transparent>=0){
			$tcolor=imagecolorsforindex($img,$transparent);
			$transparent=imagecolorallocate($thumb,$tcolor['red'],$tcolor['green'],$tcolor['blue']);
			imagefill($thumb,0,0,$transparent);
			imagecolortransparent($thumb,$transparent);
		}
	}
    imagecopyresampled($thumb,$img,0,0,0,0,$w,$h,$sw,$sh);
	imagepng($thumb,$dest);
    imagedestroy($thumb);
    imagedestroy($img);
    return true;
}
?>