<?php
if(!isset($config))
	die();

function start_session(){ // start the session and determine what we are doing
	global $config;
	session_name($config['site']['cookie']);
	session_start();
	if(isset($_GET['logout']))
		logout();
	if(isset($_GET['from']))
		upload(strip($_GET['from']),'fail');
	elseif(isset($_POST['nonce']))
		switch($_POST['nonce']){
			case nonce('login'):
				login();
				break;
			case nonce('signup'):
				signup();
				break;
			case nonce('change'):
				change();
				break;
			case nonce('edit'):
				edit();
			case nonce('upload'):
				upload();
				break;
			case nonce('caption'):
				caption();
				break;
			default:
				fail('Please try again.');
		}
	elseif(isset($_GET['activate']))
		activate();
	elseif(isset($_COOKIE[$config['site']['remember']])&&!logged_in())
		instant_login($_COOKIE[$config['site']['remember']]);
	if(isset($_GET['image']))
		image();
	if($_GET['page']=='stats')
		get_stats();
	if($_GET['page']=='delete')
		delete_image();
}

function fail($message,$to=''){ // fails forward
	$_SESSION['message']=$message;
	@header('Location: '.($to!=''?$to:$_SERVER['HTTP_REFERER']));
	die();
}

function errors(){ // displays errors
	$message=$_SESSION['message'];
	unset($_SESSION['message']);
	return $message?'<p class="error">'.$message.'</p>':'';
}

function signup(){ // processes the registration form
	global $config,$db;
	$name=strtolower($_POST['name']);
	if(strlen($name)<$config['username']['min'])
		fail("Username must be at least ".$config['username']['min']." characters.");
	if(strlen($name)>$config['username']['max'])
		fail("Username cannot be more than ".$config['username']['max']." characters.");
	if(!is_username($name))
		fail("Username can only contain alphanumeric characters.");
	$id=$db->get_var($db->prepare("SELECT id FROM users WHERE name=%s",$name));
	if($id)
		fail($name." is already in use.");
	if($_POST['password']!=$_POST['confirm-password'])
		fail("Password did not match confirmation.");
	$password=$_POST['password'];
	if(strlen($password)<$config['password']['min'])
		fail("Password must be at least ".$config['password']['min']." characters.");
	if(strtolower($_POST['email'])!=strtolower($_POST['confirm-email']))
		fail("E-mail did not match confirmation.");
	$email=strtolower($_POST['email']);
	if(!is_email($email))
		fail("E-mail address does not appear to be valid.");
	$password=md5(sha1($name).$config['password']['salt'].sha1($password));
	$result=$db->insert("users",array('name'=>$name,'password'=>$password,'email'=>$email));
	if(!$result)
		fail("Something went wrong.");
	$id=$db->insert_id;
	do_action('signup',array('id'=>$id,'name'=>$name,'email'=>$email));
	fail('Success! '.is_set('signup','require_activation')?'Please check your e-mail to activate your account.':'Your account has been created.');
}

function login(){ // processes the login form
	global $config,$db;
	if(!isset($_POST['name'])||!isset($_POST['password']))
		die();
	$name=strtolower($_POST['name']);
	$password=$_POST['password'];
	if(strlen($name)<$config['username']['min']||strlen($name)>$config['username']['max']||strlen($password)<$config['password']['min'])
		fail("Invalid username or password."); // invalid username or password length 
	if(!is_username($name))
		fail("Invalid username or password."); // invalid character in username
	$password=md5(sha1($name).$config['password']['salt'].sha1($password));
	$result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE name=%s AND password=%s",$name,$password));
	if(!$result)
		fail("Invalid username or password."); // invalid username or password
	else
		if(get_meta('activate',$result->id))
			fail("You have not activated your account."); // account not activated
	session_regenerate_id();
	$_SESSION['thumbprint']=nonce(session_id().'thumbprint',86400);
	$_SESSION['user']['id']=$result->id;
	$_SESSION['user']['name']=$result->name;
	$_SESSION['user']['email']=$result->email;
	setcookie($config['site']['remember'],$result->name.'-'.nonce($result->name,604800),time()+604800);
	$redirect=(isset($_SESSION['redirect'])?$_SESSION['redirect']:'/');
	unset($_SESSION['redirect']);
	do_action('login',$_SESSION['user']);
	header('Location: '.$redirect);
	die();
}

function instant_login($creds){ // instantly logs in if valid cookie exists
	global $config,$db;
	$creds=explode("-",$creds);
	if($creds[1]==nonce($creds[0],604800)){
		$result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE name=%s",$creds[0]));
		if(!$result){
			setcookie($config['site']['remember']);
			return;
		}
		session_regenerate_id();
		$_SESSION['thumbprint']=nonce(session_id().'thumbprint',86400);
		$_SESSION['user']['id']=$result->id;
		$_SESSION['user']['name']=$result->name;
		$_SESSION['user']['email']=$result->email;
		do_action('login',$_SESSION['user']);
		return;
	}
	setcookie($config['site']['remember']);
}

function logout(){ // destroys the session to logout
	global $config;
	do_action('logout',$_SESSION['user']);
	session_unset();
	session_destroy();
	setcookie($config['site']['remember']);
	header('Location: /');
	die();
}

function activate(){ // process account and email activation
	global $config,$db;
	$key=$_GET['activate'];
	if(strlen($key)<>32||ereg("[^a-f0-9]",$key))
		fail('Invalid activation key.',$config['pages']['login']);
	if($id=get_meta_id('activate',$key)){
		$db->query("DELETE FROM usermeta WHERE user=$id AND `key`='activate'");
		$result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE id=%d",$id));
		do_action('activate',array('id'=>$result->id,'name'=>$result->name,'email'=>$result->email));
		fail('Your account has been activated.',$config['pages']['login']);
	}
	$id=get_meta_id('confirm_email',$key);
	if($email=get_meta('new_email',$id)){
		$db->update('users',array('email'=>$email),array('id'=>$id));
		$db->query("DELETE FROM usermeta WHERE user=$id AND `key` IN ('confirm_email','new_email')");
		$result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE id=%d",$id));
		if(logged_in()&&$_SESSION['user']['id']==$result->id)
			$_SESSION['user']['email']=$email;
		do_action('new_email',array('id'=>$result->id,'name'=>$result->name,'email'=>$result->email,'new_email'=>$email));
		fail('Your e-mail address has been changed.',logged_in()?$config['pages']['manage']:$config['pages']['login']);
	}
	fail('Invalid activation key.',$config['pages']['login']);
}

function change(){ // processes the password recovery form
	global $config,$db;
	if($_POST['login']!=''){
		$user=strtolower($_POST['login']);
		if(!is_username($user)||(is_username($user)&&(strlen($user)<$config['username']['min']||strlen($user)>$config['username']['max'])))
			fail('Invalid username.');
		if(!($result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE name=%s",$user))))
			fail("Invalid username.");
		else
			if(get_meta('activate',$result->id))
				fail("You have not activated your account.");
		$key=md5(uniqid(rand(),true));
		if(!$db->update("usermeta",array('value'=>$key),array('user'=>$result->id,'key'=>'lost_password')))
			if(!$db->insert("usermeta",array('user'=>$result->id,'key'=>'lost_password','value'=>$key)))
				fail('Something went wrong.');
		if(sendmail($result->email,'Your password for '.$config['site']['name'],parse_template('email/change-password',array('link'=>$config['pages']['change'].'/'.$key),false))){
			do_action('lost_password',array('id'=>$result->id,'name'=>$result->name,'email'=>$result->email));
			fail('Please check your email for instructions on changing your password.');
		}
		fail('Something went wrong.');
	}
	elseif(isset($_POST['name'])&&isset($_POST['password'])&&isset($_POST['confirm-password'])&&isset($_GET['key'])){
		$key=$_GET['key'];
		if(strlen($key)<>32||ereg("[^a-f0-9]",$key))
			fail('Invalid key.');
		$name=strtolower($_POST['name']);
		if(!is_username($name)||strlen($name)<$config['username']['min']||strlen($name)>$config['username']['max'])
			fail('Invalid username.');
		if($_POST['password']!=$_POST['confirm-password'])
			fail("Password did not match confirmation.");
		$password=$_POST['password'];
		if(strlen($password)<$config['password']['min'])
			fail("Password must be at least ".$config['password']['min']." characters.");
		if(!($result=$db->get_row($db->prepare("SELECT id, name, email FROM users WHERE name=%s",$name))))
			fail("Invalid username.");
		if($key!=get_meta('lost_password',$result->id)||$result->id!=get_meta_id('lost_password',$key))
			fail('Invalid key.');
		$password=md5(sha1($name).$config['password']['salt'].sha1($password));
		$db->update("users",array('password'=>$password),array('id'=>$result->id));
		$db->query($db->prepare("DELETE FROM usermeta WHERE user=%d AND `key`='lost_password'",$result->id));
		do_action('new_password',array('id'=>$result->id,'name'=>$result->name,'email'=>$result->email));
		fail('Your new password has been saved.',$config['pages']['login']);
	}
	else fail('Please complete all fields.');
}

function is_email($str){ // returns true if email is a valid format
	return ereg("^[a-z0-9,!#\$%&'\*\+/=\?\^_`\{\|}~-]+(\.[a-z0-9,!#\$%&'\*\+/=\?\^_`\{\|}~-]+)*@([a-z0-9]([a-z0-9-]*[a-z0-9])?\.)+([a-z]{2}|com|org|net|gov|mil|biz|tel|info|mobi|name|aero|jobs|museum)$",$str);
}

function is_username($str){ // returns true if the username contains only alphanumeric characters
	return !ereg("[^a-z0-9.]",$str);
}

function valid_session(){ // returns true if the session is valid
	return ($_SESSION['thumbprint']==nonce(session_id().'thumbprint',86400));
}

function logged_in(){ // returns true if the user is logged in
	return (valid_session()&&isset($_SESSION['user']['id'])&&isset($_SESSION['user']['name'])&&isset($_SESSION['user']['email']));
}

function require_login(){ // allow only authenticated users
	global $config;
	if(!logged_in()){
		$_SESSION['redirect']=$_SERVER['REQUEST_URI'];
		fail('You must log in to access this page.',$config['pages']['login']);
	}
}

function edit(){ // processes the change of email form
	global $config,$db;
	require_login();
	if(strtolower($_POST['email'])!=strtolower($_POST['confirm-email']))
		fail("E-mail did not match confirmation.");
	$email=strtolower($_POST['email']);
	if($email==$_SESSION['user']['email'])
		fail("That is already set as your e-mail address.");
	if(!is_email($email))
		fail("E-mail address does not appear to be valid.");
	$key=md5(uniqid(rand(),true));
	if($db->update("usermeta",array('value'=>$email),array('user'=>$_SESSION['user']['id'],'key'=>'new_email')))
		$db->update("usermeta",array('value'=>$key),array('user'=>$_SESSION['user']['id'],'key'=>'confirm_email'));
	else{
		$db->insert("usermeta",array('user'=>$_SESSION['user']['id'],'key'=>'new_email','value'=>$email));
		$db->insert("usermeta",array('user'=>$_SESSION['user']['id'],'key'=>'confirm_email','value'=>$key));
	}
	if(sendmail($email,'Account Change at '.$config['site']['name'],parse_template('email/confirm-email',array('email'=>$email,'link'=>$config['pages']['activate'].'/'.$key),false))){
		do_action('change_email',array('id'=>$_SESSION['user']['id'],'name'=>$_SESSION['user']['name'],'email'=>$_SESSION['user']['email'],'new_email'=>$email));
		fail("Success! Please check your e-mail to confirm the new address.");
	}
	fail('Something went wrong.');
}

function login_form(){ // prints the login form
	parse_template('forms/login');
}

function signup_form(){ // prints the registration form
	parse_template('forms/signup');
}

function password_form(){ // prints the recover/change password form
	if(isset($_GET['key']))
		parse_template('forms/change-password');
	else
		parse_template('forms/recover-password');
}

function account_form(){ // prints the change e-mail form
	parse_template('forms/change-email');
}

add_action('start','start_session',20);
?>